@@ -112,6 +112,10 @@ The DONE message indicates that ghostscript has finished processing.
 #include <signal.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkprivate.h>
+#ifdef HAVE_XINERAMA
+#  include <X11/extensions/Xinerama.h>
+#  include <gdk/gdkx.h>
+#endif /* HAVE_XINERAMA */
 #include <X11/Intrinsic.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -1208,6 +1212,7 @@ check_filecompressed (GtkGS *gs)
 	FILE *file;
 	gchar buf[1024];
 	gchar *filename, *filename_unc, *filename_err, *cmdline;
+        gint fd;
         const gchar *cmd;
 
 	cmd = NULL;
@@ -1230,8 +1235,22 @@ check_filecompressed (GtkGS *gs)
 
 	/* do the decompression */
 	filename = ggv_quote_filename (gs->gs_filename);
-	filename_unc = tempnam (g_get_tmp_dir(), "unc");
-	filename_err = tempnam (g_get_tmp_dir(), "err");
+	filename_unc = g_strconcat (g_get_tmp_dir(), "/uncXXXXXX", NULL);
+        fd = mkstemp (filename_unc);
+        if (fd < 0) {
+                g_free (filename_unc);
+                return filename;
+        }
+        close (fd);
+	filename_err = g_strconcat (g_get_tmp_dir(), "/errXXXXXX", NULL);
+        fd = mkstemp (filename_err);
+        if (fd < 0) {
+                g_free (filename_unc);
+                g_free (filename_err);
+                return filename;
+        }
+        close (fd);
+                
 	cmdline = g_strdup_printf ("%s %s >%s 2>%s", cmd,
 				   filename, filename_unc, filename_err);
 	if ((system (cmdline) == 0)
@@ -1275,6 +1294,7 @@ check_pdf (GtkGS *gs)
 {
 	FILE *file;
 	gchar buf[1024], *filename;
+        gint fd;
 
 	/* use uncompressed file as input if necessary */
 	filename = (gs->gs_filename_unc ? gs->gs_filename_unc
@@ -1284,10 +1304,24 @@ check_pdf (GtkGS *gs)
 	    && (fread (buf, sizeof(char), 5, file) == 5)
 	    && (strncmp (buf, "%PDF-", 5) == 0)) {
 		/* we found a PDF file */
-		gchar *fname, *filename_dsc, *filename_err, *cmd, *cmdline;
+		gchar *fname, *filename_dsc, *filename_err, *cmd, *cmdline; 
+                filename_dsc = g_strconcat (g_get_tmp_dir(), "/dscXXXXXX", NULL);
+                fd = mkstemp (filename_dsc);
+                if (fd < 0) {
+                        g_free (filename_dsc);
+                        return gs->gs_filename;
+                }
+                close (fd);
+                filename_err = g_strconcat (g_get_tmp_dir(), "/errXXXXXX", NULL);
+                fd = mkstemp (filename_err);
+                if (fd < 0) {
+                        g_free (filename_dsc);
+                        g_free (filename_err);
+                        return gs->gs_filename;
+                }
+                close (fd);
+                
 		fname = ggv_quote_filename (filename);
-		filename_dsc = tempnam (g_get_tmp_dir(), "dsc");
-		filename_err = tempnam (g_get_tmp_dir(), "err");
 		cmd = g_strdup_printf (gtk_gs_defaults_get_scan_pdf_cmd(),
                                        fname, filename_dsc);
 		g_free (fname);
@@ -1330,13 +1364,41 @@ check_pdf (GtkGS *gs)
 static float
 compute_xdpi(void)
 {
+#ifndef HAVE_XINERAMA
         return 25.4 * gdk_screen_width()/gdk_screen_width_mm();
+#else
+        Display *dpy;
+        dpy = (Display *) GDK_DISPLAY ();
+        if (XineramaIsActive (dpy)) {
+                int num_heads;
+                XineramaScreenInfo *head_info;
+                head_info = (XineramaScreenInfo *) XineramaQueryScreens (dpy, &num_heads);
+                /* fake it with dimensions of the first head for now */
+                return 25.4 * head_info[0].width/gdk_screen_width_mm();
+        } else {
+                return 25.4 * gdk_screen_width()/gdk_screen_width_mm();
+        }
+#endif /* HAVE_XINERAMA */
 }
-
+ 
 static float
 compute_ydpi(void)
 {
+#ifndef HAVE_XINERAMA
         return 25.4 * gdk_screen_height()/gdk_screen_height_mm();
+#else
+        Display *dpy;
+        dpy = (Display *) GDK_DISPLAY ();
+        if (XineramaIsActive (dpy)) {
+                int num_heads;
+                XineramaScreenInfo *head_info;
+                head_info = (XineramaScreenInfo *) XineramaQueryScreens (dpy, &num_heads);
+                /* fake it with dimensions of the first head for now */
+                return 25.4 * head_info[0].height/gdk_screen_height_mm();
+        } else {
+                return 25.4 * gdk_screen_height()/gdk_screen_height_mm();
+        }
+#endif /* HAVE_XINERAMA */
 }
 
 /* Compute new size of window, sets xdpi and ydpi if necessary.
@@ -1837,7 +1899,8 @@ gtk_gs_set_page_size(GtkGS *gs, gint new_pagesize, gint pageid)
         g_return_val_if_fail(GTK_IS_GS(gs), FALSE);
 
         if(new_pagesize == -1) {
-                new_pagesize = gs->default_size;
+                if(gs->default_size > 0)
+                        new_pagesize = gs->default_size;
                 if (!gs->override_size && gs->doc) {
                         /* If we have a document:
                            We use -- the page size (if specified)
@@ -1874,7 +1937,7 @@ gtk_gs_set_page_size(GtkGS *gs, gint new_pagesize, gint pageid)
                 new_urx = papersizes[new_pagesize].width;
                 new_ury = papersizes[new_pagesize].height;
         }
-        else if ((gs->doc->epsf || new_pagesize == -1)) { /* epsf or bbox */
+        else if ((gs->doc->epsf && !gs->override_size) || new_pagesize == -1) { /* epsf or bbox */
                 if((gs->doc->pages) &&
                    (gs->doc->pages[pageid].boundingbox[URX] > gs->doc->pages[pageid].boundingbox[LLX]) &&
                    (gs->doc->pages[pageid].boundingbox[URY] > gs->doc->pages[pageid].boundingbox[LLY])) {
@@ -1906,6 +1969,11 @@ gtk_gs_set_page_size(GtkGS *gs, gint new_pagesize, gint pageid)
                 }
         }
 
+        if(new_urx <= new_llx)
+                new_urx = papersizes[8].width;
+        if(new_ury <= new_lly)
+                new_ury = papersizes[8].height;
+
         /* If bounding box changed, setup for new size. */
                 /* gtk_gs_disable_interpreter (gs); */
         if ((new_llx != gs->llx) || (new_lly != gs->lly) ||
